@@ -10,8 +10,8 @@ package Text::Table::Boxed;
 use base 'Text::Table';
 
 { no strict 'refs'; ${__PACKAGE__."::VER"."SION"} = 997.999; }
-our $VERSION = '1.000'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
-our $DATE = '2024-12-11'; # DATE from Dist::Zilla::Plugin::OurDate
+our $VERSION = '1.001'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
+our $DATE = '2024-12-12'; # DATE from Dist::Zilla::Plugin::OurDate
 
 use Carp;
 use Scalar::Util qw/reftype/;
@@ -148,8 +148,8 @@ btw dvis 'BEFORE PIC-LEN-ADJ: $num_cols $pic_cols @sepsegs\n@top_rule\n@mid_rule
     #             ^^^^^^^^^^ #replicate or delete these
     #             $#-2   $#-1  $#
     if ($pic_cols < $num_cols) {
-      croak "Picture must have three columns if data has >= three columns\n"
-        if $pic_cols < 3; # no separator to replicate
+      croak "Picture must have at least two columns\n"
+        if $pic_cols < 2; # no separator to replicate
       foreach (\@sepsegs, @top_rule, @mid_rule, @bot_rule) {
         splice @$_, $#$_, 0, @$_[$#$_-2..$#$_-1];
       }
